@@ -3,6 +3,7 @@
 import re
 from . import simple
 from . import snowball
+import annif
 
 _analyzers = {}
 
@@ -31,3 +32,10 @@ def get_analyzer(analyzerspec):
 
 register_analyzer(simple.SimpleAnalyzer)
 register_analyzer(snowball.SnowballAnalyzer)
+
+# Optional analyzers
+try:
+    from . import voikko
+    register_analyzer(voikko.VoikkoAnalyzer)
+except ImportError:
+    annif.logger.debug("voikko not available, not enabling voikko analyzer")
