@@ -11,18 +11,19 @@ class AnnifBackend(metaclass=abc.ABCMeta):
     name = None
     needs_subject_index = False
 
-    DEFAULT_PARAMS = {'limit': 100}
+    DEFAULT_PARAMETERS = {'limit': 100}
 
-    def __init__(self, backend_id, config_params, datadir):
+    def __init__(self, backend_id, config_params, project):
         """Initialize backend with specific parameters. The
         parameters are a dict. Keys and values depend on the specific
         backend type."""
         self.backend_id = backend_id
-        self.datadir = datadir
         self.config_params = config_params
+        self.project = project
+        self.datadir = project.datadir
 
     def default_params(self):
-        return self.DEFAULT_PARAMS
+        return self.DEFAULT_PARAMETERS
 
     @property
     def params(self):
@@ -31,29 +32,39 @@ class AnnifBackend(metaclass=abc.ABCMeta):
         params.update(self.config_params)
         return params
 
-    def train(self, corpus, project):
-        """train the model on the given document or subject corpus"""
+    def _get_backend_params(self, params):
+        backend_params = dict(self.params)
+        if params is not None:
+            backend_params.update(params)
+        return backend_params
+
+    def _train(self, corpus, params):
+        """This method can be overridden by backends. It implements
+        the train functionality, with pre-processed parameters."""
         pass  # default is to do nothing, subclasses may override
 
+    def train(self, corpus, params=None):
+        """Train the model on the given document or subject corpus."""
+        beparams = self._get_backend_params(params)
+        return self._train(corpus, params=beparams)
+
     def initialize(self):
         """This method can be overridden by backends. It should cause the
         backend to pre-load all data it needs during operation."""
         pass
 
     @abc.abstractmethod
-    def _suggest(self, text, project, params):
+    def _suggest(self, text, params):
         """This method should implemented by backends. It implements
         the suggest functionality, with pre-processed parameters."""
         pass  # pragma: no cover
 
-    def suggest(self, text, project, params=None):
+    def suggest(self, text, params=None):
         """Suggest subjects for the input text and return a list of subjects
         represented as a list of SubjectSuggestion objects."""
+        beparams = self._get_backend_params(params)
         self.initialize()
-        beparams = dict(self.params)
-        if params:
-            beparams.update(params)
-        return self._suggest(text, project, params=beparams)
+        return self._suggest(text, params=beparams)
 
     def debug(self, message):
         """Log a debug message from this backend"""
@@ -72,6 +83,12 @@ class AnnifLearningBackend(AnnifBackend):
     """Base class for Annif backends that can perform online learning"""
 
     @abc.abstractmethod
-    def learn(self, corpus, project):
-        """further train the model on the given document or subject corpus"""
+    def _learn(self, corpus, params):
+        """This method should implemented by backends. It implements the learn
+        functionality, with pre-processed parameters."""
         pass  # pragma: no cover
+
+    def learn(self, corpus, params=None):
+        """Further train the model on the given document or subject corpus."""
+        beparams = self._get_backend_params(params)
+        return self._learn(corpus, params=beparams)
