@@ -12,19 +12,21 @@ class DummyBackend(backend.AnnifLearningBackend):
     label = 'dummy'
 
     def default_params(self):
-        return backend.AnnifBackend.DEFAULT_PARAMS
+        return backend.AnnifBackend.DEFAULT_PARAMETERS
 
     def initialize(self):
         self.initialized = True
 
-    def _suggest(self, text, project, params):
+    def _suggest(self, text, params):
         score = float(params.get('score', 1.0))
+        notation = params.get('notation', None)
         return ListSuggestionResult([SubjectSuggestion(uri=self.uri,
                                                        label=self.label,
+                                                       notation=notation,
                                                        score=score)],
-                                    project.subjects)
+                                    self.project.subjects)
 
-    def learn(self, corpus, project):
+    def _learn(self, corpus, params):
         # in this dummy backend we "learn" by picking up the URI and label
         # of the first subject of the first document in the learning set
         # and using that in subsequent analysis results
