@@ -33,7 +33,7 @@ class FastTextBackend(mixins.ChunkingBackend, backend.AnnifBackend):
         't': float
     }
 
-    DEFAULT_PARAMS = {
+    DEFAULT_PARAMETERS = {
         'dim': 100,
         'lr': 0.25,
         'epoch': 5,
@@ -47,9 +47,9 @@ class FastTextBackend(mixins.ChunkingBackend, backend.AnnifBackend):
     _model = None
 
     def default_params(self):
-        params = backend.AnnifBackend.DEFAULT_PARAMS.copy()
-        params.update(mixins.ChunkingBackend.DEFAULT_PARAMS)
-        params.update(self.DEFAULT_PARAMS)
+        params = backend.AnnifBackend.DEFAULT_PARAMETERS.copy()
+        params.update(mixins.ChunkingBackend.DEFAULT_PARAMETERS)
+        params.update(self.DEFAULT_PARAMETERS)
         return params
 
     def initialize(self):
@@ -69,11 +69,10 @@ class FastTextBackend(mixins.ChunkingBackend, backend.AnnifBackend):
     def _id_to_label(subject_id):
         return "__label__{:d}".format(subject_id)
 
-    @staticmethod
-    def _label_to_subject(project, label):
+    def _label_to_subject(self, label):
         labelnum = label.replace('__label__', '')
         subject_id = int(labelnum)
-        return project.subjects[subject_id]
+        return self.project.subjects[subject_id]
 
     def _write_train_file(self, doc_subjects, filename):
         with open(filename, 'w', encoding='utf-8') as trainfile:
@@ -85,20 +84,19 @@ class FastTextBackend(mixins.ChunkingBackend, backend.AnnifBackend):
                 else:
                     self.warning('no labels for document "{}"'.format(doc))
 
-    @staticmethod
-    def _normalize_text(project, text):
-        return ' '.join(project.analyzer.tokenize_words(text))
+    def _normalize_text(self, text):
+        return ' '.join(self.project.analyzer.tokenize_words(text))
 
-    def _create_train_file(self, corpus, project):
+    def _create_train_file(self, corpus):
         self.info('creating fastText training file')
 
         doc_subjects = collections.defaultdict(set)
 
         for doc in corpus.documents:
-            text = self._normalize_text(project, doc.text)
+            text = self._normalize_text(doc.text)
             if text == '':
                 continue
-            doc_subjects[text] = [project.subjects.by_uri(uri)
+            doc_subjects[text] = [self.project.subjects.by_uri(uri)
                                   for uri in doc.uris]
 
         annif.util.atomic_save(doc_subjects,
@@ -106,32 +104,37 @@ class FastTextBackend(mixins.ChunkingBackend, backend.AnnifBackend):
                                self.TRAIN_FILE,
                                method=self._write_train_file)
 
-    def _create_model(self):
+    def _create_model(self, params):
         self.info('creating fastText model')
         trainpath = os.path.join(self.datadir, self.TRAIN_FILE)
         modelpath = os.path.join(self.datadir, self.MODEL_FILE)
         params = {param: self.FASTTEXT_PARAMS[param](val)
-                  for param, val in self.params.items()
+                  for param, val in params.items()
                   if param in self.FASTTEXT_PARAMS}
+        self.debug('Model parameters: {}'.format(params))
         self._model = fastText.train_supervised(trainpath, **params)
         self._model.save_model(modelpath)
 
-    def train(self, corpus, project):
-        if corpus.is_empty():
-            raise NotSupportedException('training backend {} with no documents'
-                                        .format(self.backend_id))
-        self._create_train_file(corpus, project)
-        self._create_model()
+    def _train(self, corpus, params):
+        if corpus != 'cached':
+            if corpus.is_empty():
+                raise NotSupportedException(
+                    'training backend {} with no documents' .format(
+                        self.backend_id))
+            self._create_train_file(corpus)
+        else:
+            self.info("Reusing cached training data from previous run.")
+        self._create_model(params)
 
-    def _predict_chunks(self, chunktexts, project, limit):
+    def _predict_chunks(self, chunktexts, limit):
         return self._model.predict(list(
-            filter(None, [self._normalize_text(project, chunktext)
+            filter(None, [self._normalize_text(chunktext)
                           for chunktext in chunktexts])), limit)
 
-    def _suggest_chunks(self, chunktexts, project):
-        limit = int(self.params['limit'])
+    def _suggest_chunks(self, chunktexts, params):
+        limit = int(params['limit'])
         chunklabels, chunkscores = self._predict_chunks(
-            chunktexts, project, limit)
+            chunktexts, limit)
         label_scores = collections.defaultdict(float)
         for labels, scores in zip(chunklabels, chunkscores):
             for label, score in zip(labels, scores):
@@ -142,9 +145,10 @@ class FastTextBackend(mixins.ChunkingBackend, backend.AnnifBackend):
 
         results = []
         for score, label in best_labels[:limit]:
-            subject = self._label_to_subject(project, label)
+            subject = self._label_to_subject(label)
             results.append(SubjectSuggestion(
                 uri=subject[0],
                 label=subject[1],
+                notation=subject[2],
                 score=score / len(chunktexts)))
-        return ListSuggestionResult(results, project.subjects)
+        return ListSuggestionResult(results, self.project.subjects)
