@@ -6,6 +6,20 @@ import warnings
 import numpy as np
 from sklearn.metrics import precision_score, recall_score, f1_score
 from sklearn.metrics import label_ranking_average_precision_score
+from annif.exception import NotSupportedException
+
+
+def filter_pred_top_k(preds, limit):
+    """filter a 2D prediction vector, retaining only the top K suggestions
+    for each individual prediction; the rest will be set to zeros"""
+
+    masks = []
+    for pred in preds:
+        mask = np.zeros_like(pred, dtype=np.bool)
+        top_k = np.argsort(pred)[::-1][:limit]
+        mask[top_k] = True
+        masks.append(mask)
+    return preds * np.array(masks)
 
 
 def true_positives(y_true, y_pred):
@@ -30,7 +44,6 @@ def precision_at_k_score(y_true, y_pred, limit):
     """calculate the precision at K, i.e. the number of relevant items
     among the top K predicted ones"""
     scores = []
-    origlimit = limit
     for true, pred in zip(y_true, y_pred):
         order = pred.argsort()[::-1]
         orderlimit = min(limit, np.count_nonzero(pred))
@@ -108,6 +121,8 @@ class EvaluationBatch:
                     y_true, y_pred_binary, average='micro')
                 results['F1 score (microavg)'] = f1_score(
                     y_true, y_pred_binary, average='micro')
+            results['F1@5'] = f1_score(
+                y_true, filter_pred_top_k(y_pred, 5) > 0.0, average='samples')
             results['NDCG'] = ndcg_score(y_true, y_pred)
             results['NDCG@5'] = ndcg_score(y_true, y_pred, limit=5)
             results['NDCG@10'] = ndcg_score(y_true, y_pred, limit=10)
@@ -134,6 +149,9 @@ class EvaluationBatch:
         different metrics. The set of metrics can be either 'all' or
         'simple'."""
 
+        if not self._samples:
+            raise NotSupportedException("cannot evaluate empty corpus")
+
         y_true = np.array([gold_subjects.as_vector(self._subject_index)
                            for hits, gold_subjects in self._samples])
         y_pred = np.array([hits.vector
