@@ -1,17 +1,18 @@
 """Unit tests for the PAV backend in Annif"""
 
+import logging
 import pytest
 import annif.backend
 import annif.corpus
 from annif.exception import NotSupportedException
 
 
-def test_pav_default_params(datadir, document_corpus, project):
+def test_pav_default_params(document_corpus, project):
     pav_type = annif.backend.get_backend("pav")
     pav = pav_type(
         backend_id='pav',
         config_params={},
-        datadir=str(datadir))
+        project=project)
 
     expected_default_params = {
         'min-docs': 10,
@@ -26,7 +27,7 @@ def test_pav_train(app, datadir, tmpdir, project):
     pav = pav_type(
         backend_id='pav',
         config_params={'limit': 50, 'min-docs': 2, 'sources': 'dummy-fi'},
-        datadir=str(datadir))
+        project=project)
 
     tmpfile = tmpdir.join('document.tsv')
     tmpfile.write("dummy\thttp://example.org/dummy\n" +
@@ -35,32 +36,40 @@ def test_pav_train(app, datadir, tmpdir, project):
     document_corpus = annif.corpus.DocumentFile(str(tmpfile))
 
     with app.app_context():
-        pav.train(document_corpus, project)
+        pav.train(document_corpus)
     assert datadir.join('pav-model-dummy-fi').exists()
     assert datadir.join('pav-model-dummy-fi').size() > 0
 
 
-def test_pav_train_nodocuments(tmpdir, datadir, project):
+def test_pav_train_cached(project):
     pav_type = annif.backend.get_backend("pav")
     pav = pav_type(
         backend_id='pav',
         config_params={'limit': 50, 'min-docs': 2, 'sources': 'dummy-fi'},
-        datadir=str(datadir))
+        project=project)
 
-    empty_file = tmpdir.ensure('empty.tsv')
-    empty_document_corpus = annif.corpus.DocumentFile(str(empty_file))
+    with pytest.raises(NotSupportedException):
+        pav.train("cached")
+
+
+def test_pav_train_nodocuments(project, empty_corpus):
+    pav_type = annif.backend.get_backend("pav")
+    pav = pav_type(
+        backend_id='pav',
+        config_params={'limit': 50, 'min-docs': 2, 'sources': 'dummy-fi'},
+        project=project)
 
     with pytest.raises(NotSupportedException) as excinfo:
-        pav.train(empty_document_corpus, project)
+        pav.train(empty_corpus)
     assert 'training backend pav with no documents' in str(excinfo.value)
 
 
-def test_pav_initialize(app, datadir):
+def test_pav_initialize(app, project):
     pav_type = annif.backend.get_backend("pav")
     pav = pav_type(
         backend_id='pav',
         config_params={'limit': 50, 'min-docs': 2, 'sources': 'dummy-fi'},
-        datadir=str(datadir))
+        project=project)
 
     assert pav._models is None
     with app.app_context():
@@ -71,19 +80,42 @@ def test_pav_initialize(app, datadir):
         pav.initialize()
 
 
-def test_pav_suggest(app, datadir, project):
+def test_pav_suggest(app, project):
     pav_type = annif.backend.get_backend("pav")
     pav = pav_type(
         backend_id='pav',
         config_params={'limit': 50, 'min-docs': 2, 'sources': 'dummy-fi'},
-        datadir=str(datadir))
+        project=project)
 
     results = pav.suggest("""Arkeologiaa sanotaan joskus myös
         muinaistutkimukseksi tai muinaistieteeksi. Se on humanistinen tiede
         tai oikeammin joukko tieteitä, jotka tutkivat ihmisen menneisyyttä.
         Tutkimusta tehdään analysoimalla muinaisjäännöksiä eli niitä jälkiä,
         joita ihmisten toiminta on jättänyt maaperään tai vesistöjen
-        pohjaan.""", project)
+        pohjaan.""")
 
     assert len(pav._models['dummy-fi']) == 1
     assert len(results) > 0
+
+
+def test_pav_train_params(app, tmpdir, project, caplog):
+    logger = annif.logger
+    logger.propagate = True
+
+    pav_type = annif.backend.get_backend("pav")
+    pav = pav_type(
+        backend_id='pav',
+        config_params={'limit': 50, 'min-docs': 2, 'sources': 'dummy-fi'},
+        project=project)
+
+    tmpfile = tmpdir.join('document.tsv')
+    tmpfile.write("dummy\thttp://example.org/dummy\n" +
+                  "another\thttp://example.org/dummy\n" +
+                  "none\thttp://example.org/none")
+    document_corpus = annif.corpus.DocumentFile(str(tmpfile))
+    params = {'min-docs': 5}
+
+    with app.app_context(), caplog.at_level(logging.DEBUG):
+        pav.train(document_corpus, params)
+    parameters_spec = 'creating PAV model for source dummy-fi, min_docs=5'
+    assert parameters_spec in caplog.text
