@@ -8,20 +8,12 @@ import pytest
 import unittest.mock
 
 
-@pytest.fixture(scope='module')
-def project(document_corpus, subject_index):
-    proj = unittest.mock.Mock()
-    proj.analyzer = annif.analyzer.get_analyzer('snowball(finnish)')
-    proj.subjects = subject_index
-    return proj
-
-
-def test_tfidf_default_params(datadir, project):
+def test_tfidf_default_params(project):
     tfidf_type = annif.backend.get_backend("tfidf")
     tfidf = tfidf_type(
         backend_id='tfidf',
         config_params={},
-        datadir=str(datadir))
+        project=project)
 
     expected_default_params = {
         'limit': 100  # From AnnifBackend class
@@ -36,27 +28,27 @@ def test_tfidf_train(datadir, document_corpus, project):
     tfidf = tfidf_type(
         backend_id='tfidf',
         config_params={'limit': 10},
-        datadir=str(datadir))
+        project=project)
 
-    tfidf.train(document_corpus, project)
+    tfidf.train(document_corpus)
     assert len(tfidf._index) > 0
     assert datadir.join('tfidf-index').exists()
     assert datadir.join('tfidf-index').size() > 0
 
 
-def test_tfidf_suggest(datadir, project):
+def test_tfidf_suggest(project):
     tfidf_type = annif.backend.get_backend("tfidf")
     tfidf = tfidf_type(
         backend_id='tfidf',
         config_params={'limit': 10},
-        datadir=str(datadir))
+        project=project)
 
     results = tfidf.suggest("""Arkeologiaa sanotaan joskus myös
         muinaistutkimukseksi tai muinaistieteeksi. Se on humanistinen tiede
         tai oikeammin joukko tieteitä, jotka tutkivat ihmisen menneisyyttä.
         Tutkimusta tehdään analysoimalla muinaisjäännöksiä eli niitä jälkiä,
         joita ihmisten toiminta on jättänyt maaperään tai vesistöjen
-        pohjaan.""", project)
+        pohjaan.""")
 
     assert len(results) == 10
     assert 'http://www.yso.fi/onto/yso/p1265' in [
@@ -64,13 +56,30 @@ def test_tfidf_suggest(datadir, project):
     assert 'arkeologia' in [result.label for result in results]
 
 
-def test_tfidf_suggest_unknown(datadir, project):
+def test_suggest_params(project):
     tfidf_type = annif.backend.get_backend("tfidf")
     tfidf = tfidf_type(
         backend_id='tfidf',
         config_params={'limit': 10},
-        datadir=str(datadir))
+        project=project)
+    params = {'limit': 3}
 
-    results = tfidf.suggest("abcdefghijk", project)  # unknown word
+    results = tfidf.suggest("""Arkeologiaa sanotaan joskus myös
+        muinaistutkimukseksi tai muinaistieteeksi. Se on humanistinen tiede
+        tai oikeammin joukko tieteitä, jotka tutkivat ihmisen menneisyyttä.
+        Tutkimusta tehdään analysoimalla muinaisjäännöksiä eli niitä jälkiä,
+        joita ihmisten toiminta on jättänyt maaperään tai vesistöjen
+        pohjaan.""", params)
+    assert len(results) == 3
+
+
+def test_tfidf_suggest_unknown(project):
+    tfidf_type = annif.backend.get_backend("tfidf")
+    tfidf = tfidf_type(
+        backend_id='tfidf',
+        config_params={'limit': 10},
+        project=project)
+
+    results = tfidf.suggest("abcdefghijk")  # unknown word
 
     assert len(results) == 0
