@@ -110,6 +110,8 @@ def test_clear_project_nonexistent_data(testdatadir, caplog):
 def test_loadvoc_tsv(testdatadir):
     with contextlib.suppress(FileNotFoundError):
         os.remove(str(testdatadir.join('projects/tfidf-fi/subjects')))
+    with contextlib.suppress(FileNotFoundError):
+        os.remove(str(testdatadir.join('projects/tfidf-fi/subjects.ttl')))
     subjectfile = os.path.join(
         os.path.dirname(__file__),
         'corpora',
@@ -120,11 +122,34 @@ def test_loadvoc_tsv(testdatadir):
     assert result.exit_code == 0
     assert testdatadir.join('vocabs/yso-fi/subjects').exists()
     assert testdatadir.join('vocabs/yso-fi/subjects').size() > 0
+    assert testdatadir.join('vocabs/yso-fi/subjects.ttl').exists()
+    assert testdatadir.join('vocabs/yso-fi/subjects.ttl').size() > 0
+
+
+def test_loadvoc_tsv_with_bom(testdatadir):
+    with contextlib.suppress(FileNotFoundError):
+        os.remove(str(testdatadir.join('projects/tfidf-fi/subjects')))
+    with contextlib.suppress(FileNotFoundError):
+        os.remove(str(testdatadir.join('projects/tfidf-fi/subjects.ttl')))
+    subjectfile = os.path.join(
+        os.path.dirname(__file__),
+        'corpora',
+        'archaeology',
+        'subjects-bom.tsv')
+    result = runner.invoke(annif.cli.cli, ['loadvoc', 'tfidf-fi', subjectfile])
+    assert not result.exception
+    assert result.exit_code == 0
+    assert testdatadir.join('vocabs/yso-fi/subjects').exists()
+    assert testdatadir.join('vocabs/yso-fi/subjects').size() > 0
+    assert testdatadir.join('vocabs/yso-fi/subjects.ttl').exists()
+    assert testdatadir.join('vocabs/yso-fi/subjects.ttl').size() > 0
 
 
 def test_loadvoc_rdf(testdatadir):
     with contextlib.suppress(FileNotFoundError):
         os.remove(str(testdatadir.join('projects/tfidf-fi/subjects')))
+    with contextlib.suppress(FileNotFoundError):
+        os.remove(str(testdatadir.join('projects/tfidf-fi/subjects.ttl')))
     subjectfile = os.path.join(
         os.path.dirname(__file__),
         'corpora',
@@ -135,6 +160,27 @@ def test_loadvoc_rdf(testdatadir):
     assert result.exit_code == 0
     assert testdatadir.join('vocabs/yso-fi/subjects').exists()
     assert testdatadir.join('vocabs/yso-fi/subjects').size() > 0
+    assert testdatadir.join('vocabs/yso-fi/subjects.ttl').exists()
+    assert testdatadir.join('vocabs/yso-fi/subjects.ttl').size() > 0
+
+
+def test_loadvoc_ttl(testdatadir):
+    with contextlib.suppress(FileNotFoundError):
+        os.remove(str(testdatadir.join('projects/tfidf-fi/subjects')))
+    with contextlib.suppress(FileNotFoundError):
+        os.remove(str(testdatadir.join('projects/tfidf-fi/subjects.ttl')))
+    subjectfile = os.path.join(
+        os.path.dirname(__file__),
+        'corpora',
+        'archaeology',
+        'yso-archaeology.ttl')
+    result = runner.invoke(annif.cli.cli, ['loadvoc', 'tfidf-fi', subjectfile])
+    assert not result.exception
+    assert result.exit_code == 0
+    assert testdatadir.join('vocabs/yso-fi/subjects').exists()
+    assert testdatadir.join('vocabs/yso-fi/subjects').size() > 0
+    assert testdatadir.join('vocabs/yso-fi/subjects.ttl').exists()
+    assert testdatadir.join('vocabs/yso-fi/subjects.ttl').size() > 0
 
 
 def test_loadvoc_nonexistent_path():
@@ -178,6 +224,45 @@ def test_train_multiple(testdatadir):
     assert testdatadir.join('projects/tfidf-fi/tfidf-index').size() > 0
 
 
+def test_train_cached(testdatadir):
+    result = runner.invoke(annif.cli.cli,
+                           ['train', '--cached', 'tfidf-fi'])
+    assert result.exception
+    assert result.exit_code == 1
+    assert 'Training tfidf project from cached data not supported.' \
+           in result.output
+
+
+def test_train_cached_with_corpus(testdatadir):
+    docfile = os.path.join(
+        os.path.dirname(__file__),
+        'corpora',
+        'archaeology',
+        'documents.tsv')
+    result = runner.invoke(annif.cli.cli,
+                           ['train', '--cached', 'tfidf-fi', docfile])
+    assert result.exception
+    assert result.exit_code == 2
+    assert 'Corpus paths cannot be given when using --cached option.' \
+           in result.output
+
+
+def test_train_param_override_algo_notsupported():
+    pytest.importorskip('annif.backend.vw_multi')
+    docfile = os.path.join(
+        os.path.dirname(__file__),
+        'corpora',
+        'archaeology',
+        'documents.tsv')
+    result = runner.invoke(
+        annif.cli.cli,
+        ['train', 'vw-multi-fi', docfile,
+         '--backend-param', 'vw_multi.algorithm=oaa'])
+    assert result.exception
+    assert result.exit_code == 1
+    assert 'Algorithm overriding not supported.' in result.output
+
+
 def test_train_nonexistent_path():
     failed_result = runner.invoke(
         annif.cli.cli, [
@@ -241,6 +326,16 @@ def test_suggest():
     assert result.exit_code == 0
 
 
+def test_suggest_with_notations():
+    result = runner.invoke(
+        annif.cli.cli,
+        ['suggest', '--backend-param', 'dummy.notation=42.42', 'dummy-fi'],
+        input='kissa')
+    assert not result.exception
+    assert result.output == "<http://example.org/dummy>\tdummy\t42.42\t1.0\n"
+    assert result.exit_code == 0
+
+
 def test_suggest_nonexistent():
     result = runner.invoke(
         annif.cli.cli,
@@ -262,6 +357,18 @@ def test_suggest_param():
     assert result.exit_code == 0
 
 
+def test_suggest_param_backend_nonexistent():
+    result = runner.invoke(
+        annif.cli.cli,
+        ['suggest', '--backend-param', 'not_a_backend.score=0.8', 'dummy-fi'],
+        input='kissa')
+    assert result.exception
+    assert 'The backend not_a_backend in CLI option ' + \
+        '"-b not_a_backend.score=0.8" not matching the project backend ' + \
+        'dummy.' in result.output
+    assert result.exit_code != 0
+
+
 def test_suggest_ensemble():
     result = runner.invoke(
         annif.cli.cli,
@@ -348,11 +455,9 @@ def test_eval_label(tmpdir):
 
 def test_eval_uri(tmpdir):
     tmpdir.join('doc1.txt').write('doc1')
-    keyfile = tmpdir.join('doc1.key').write(
-        "<http://example.org/dummy>\tdummy\n")
+    tmpdir.join('doc1.key').write("<http://example.org/dummy>\tdummy\n")
     tmpdir.join('doc2.txt').write('doc2')
-    keyfile = tmpdir.join('doc2.key').write(
-        "<http://example.org/none>\tnone\n")
+    tmpdir.join('doc2.key').write("<http://example.org/none>\tnone\n")
     tmpdir.join('doc3.txt').write('doc3')
 
     result = runner.invoke(annif.cli.cli, ['eval', 'dummy-en', str(tmpdir)])
@@ -414,6 +519,16 @@ def test_eval_docfile():
     assert result.exit_code == 0
 
 
+def test_eval_empty_file(tmpdir):
+    empty_file = tmpdir.ensure('empty.tsv')
+    failed_result = runner.invoke(
+        annif.cli.cli, [
+            'eval', 'dummy-fi', str(empty_file)])
+    assert failed_result.exception
+    assert failed_result.exit_code != 0
+    assert 'cannot evaluate empty corpus' in failed_result.output
+
+
 def test_eval_nonexistent_path():
     failed_result = runner.invoke(
         annif.cli.cli, [
