@@ -9,7 +9,7 @@ from annif.project import Access
 
 def test_create_project_wrong_access(app):
     with pytest.raises(ConfigurationException):
-        project = annif.project.AnnifProject(
+        annif.project.AnnifProject(
             'example',
             {'name': 'Example', 'language': 'en', 'access': 'invalid'},
             '.')
@@ -72,21 +72,21 @@ def test_get_project_noanalyzer(app):
     with app.app_context():
         project = annif.project.get_project('noanalyzer')
         with pytest.raises(ConfigurationException):
-            analyzer = project.analyzer
+            project.analyzer
 
 
 def test_get_project_novocab(app):
     with app.app_context():
         project = annif.project.get_project('novocab')
         with pytest.raises(ConfigurationException):
-            vocab = project.vocab
+            project.vocab
 
 
 def test_get_project_nobackend(app):
     with app.app_context():
         project = annif.project.get_project('nobackend')
         with pytest.raises(ConfigurationException):
-            backend = project.backend
+            project.backend
 
 
 def test_get_project_noname(app):
@@ -121,18 +121,18 @@ def test_get_project_default_params_fasttext(app):
         assert param in actual_params and actual_params[param] == val
 
 
-def test_get_project_invalid_config_file(app):
+def test_get_project_invalid_config_file():
     app = annif.create_app(
         config_name='annif.default_config.TestingInvalidProjectsConfig')
     with app.app_context():
         with pytest.raises(ConfigurationException):
-            project = annif.project.get_project('duplicatedvocab')
+            annif.project.get_project('duplicatedvocab')
 
 
 def test_project_load_vocabulary_tfidf(app, vocabulary, testdatadir):
     with app.app_context():
         project = annif.project.get_project('tfidf-fi')
-    project.vocab.load_vocabulary(vocabulary)
+    project.vocab.load_vocabulary(vocabulary, 'fi')
     assert testdatadir.join('vocabs/yso-fi/subjects').exists()
     assert testdatadir.join('vocabs/yso-fi/subjects').size() > 0
 
@@ -145,13 +145,11 @@ def test_project_train_tfidf(app, document_corpus, testdatadir):
     assert testdatadir.join('projects/tfidf-fi/tfidf-index').size() > 0
 
 
-def test_project_train_tfidf_nodocuments(app, tmpdir):
+def test_project_train_tfidf_nodocuments(app, empty_corpus):
     with app.app_context():
         project = annif.project.get_project('tfidf-fi')
-    empty_file = tmpdir.ensure('empty.tsv')
-    empty_document_corpus = annif.corpus.DocumentFile(str(empty_file))
     with pytest.raises(NotSupportedException) as excinfo:
-        project.train(empty_document_corpus)
+        project.train(empty_corpus)
     assert 'Cannot train tfidf project with no documents' in str(excinfo.value)
 
 
@@ -189,7 +187,7 @@ def test_project_load_vocabulary_fasttext(app, vocabulary, testdatadir):
     pytest.importorskip("annif.backend.fasttext")
     with app.app_context():
         project = annif.project.get_project('fasttext-fi')
-    project.vocab.load_vocabulary(vocabulary)
+    project.vocab.load_vocabulary(vocabulary, 'fi')
     assert testdatadir.join('vocabs/yso-fi/subjects').exists()
     assert testdatadir.join('vocabs/yso-fi/subjects').size() > 0
 
@@ -241,4 +239,4 @@ def test_project_file_not_found():
         config_name='annif.default_config.TestingNoProjectsConfig')
     with app.app_context():
         with pytest.raises(ValueError):
-            project = annif.project.get_project('dummy-en')
+            annif.project.get_project('dummy-en')
